#########################################################################
##  SPLINE.R
##  
##  This is a R program, written by
##  Bruce E. Hansen
##  Department of Economics
##  University of Wisconsin
##  behansen@wisc.edu
##  http:/www.ssc.wisc.edu/~bhansen
##  
##  This program estimates the spline transfer function model presented in
##  "How Responsive are Private Transfers to Income?
##  Evidence From a Laissez-Faire Economy"
##  by Donald Cox, Bruce Hansen, and Emmanuel Jimenez.
##  
##  The results of this program generates the estimates reported in Table 3
##  of the paper, and Figures 2 and 3.
##  
##  This program will take a long time to execute if one sets qn=0.  To speed up
##  execution (with little cost in precision), set qn=500 or even qn=100.
##  
#########################################################################

trm <- .02       # Percentile of high-income households deleted  
per <- .10       # Percentage to trim from quantiles for threshold search 
qn  <- 0         # Set qn=0  to search across all thresholds.
                 # Set qn to a positive integers to search
                 # across a fixed number of quantiles     
autop <- 0       # If set to 1, will automatically print graphs   
noinc <- 1       # If set to 1, includes dummy for no income, else set to 0 
quad <- 0        # If set to 1, includes quadratic term for non-altruistic regime 
age2 <- 0        # If set to 1, includes quadratic term in age of household head 
save <- 0        # Set to 1 to save transfer functions 


#******************* function ***************************#

report <- function(y,x,e,theta,xname){
  ee <- t(e)%*%e
  sig <- as.vector(ee/(nrow(y)-ncol(x)))
  mi <- solve(t(x)%*%x, tol=1e-040, LAPACK=FALSE)
  se <- sqrt(abs(diag(mi*sig)))
  xe <- x*(e%*%matrix(1,1,ncol(x)))
  seh <- sqrt(abs(diag(mi%*%(t(xe)%*%xe)%*%mi)))
  Txname <- format(xname,digits=4)
  Ttheta <- format(theta,digits=4)
  Tse <- format(se,digits=4) 
  Tseh <- format(seh,digits=4)
  cat ("Sum of Squared Errors               ", ee, "\n")
  cat ("Residual Standard Deviation         ", sqrt(sig), "\n")
  cat ("R-squared                           ", (1 - ee/(sum((y-mean(y))^2))), "\n")
  cat ("\n")
  cat ("Variable    ","Estimate    ","St Error    ","Het St Er    ","\n")
  cat ("____________________________________________________","\n")
  for (j in 1:nrow(theta)) cat (Txname[j]," ",Ttheta[j],"  ",Tse[j],"  ",Tseh[j],"\n")
  cat ("____________________________________________________","\n")
  cat ("\n")
  cat ("\n")
  cat ("\n")
}

gr_tr <- function(i,tr){
mtit <- paste(sample,"households")
ytit <- "Transfers (Pesos)"
xtit <- "Total Income (Pesos)"
if (smp==1){
 xtics <- cbind(0,200000) 
 ytics <- cbind(0,20000)
} 
if (smp==2){
 xtics <- cbind(0,80000)
 ytics <- cbind(0,10000)
}
x11()
plot(i,tr[,1],lty=1,col=1,type="l",xlim=xtics,ylim=ytics,ann=0)
lines(i,tr[,2],lty=2,col=2)
title(main=mtit,ylab=ytit,xlab=xtit) 
}

#********************************************************#


smp <- 1         # Set to 1 if "urban" or 2 if "rural"  

while (smp<=2){    

rname <- rbind("Region1","Region2","Region3","Region4","Region5",
               "Region6","Region7","Region8","Region9","Region10",
               "Region11","Region12","Region13")

vn <- rbind("Abroad  ","Domestic","Inkind  ","Gifts   ","Income  ",                    
            "Wages   ","Self    ","Rent    ","Interest","RetInc  ",                     
            "Dividend","Primary ","Some Hgh","High Grd","Some Col",                     
            "College ","Age     ","Female  ","Married ","Kids<1  ",                     
            "1<Kids<7","7<Kid<15","HH Size ","Both Wrk","Not Empl",                    
            "Mar Fem ","Region  ")

if (smp==1){
  rname <- rbind("Region0",rname)
  sample <- "Urban"
  dat <- read.table("urban.dat")
}
if (smp==2){ 
  sample <- "Rural"
  dat <- read.table("rural.dat")
}

abroad <- dat[,1]
domestic <- dat[,2]
inkind <- dat[,3]
gifts <- dat[,4]
totinc <- dat[,5]
retinc <- dat[,10]
yname <- "Transfers"

y <- abroad+domestic+inkind-gifts
i <- totinc+gifts-abroad-domestic-inkind-retinc

# delete observations with negative income #
dat <- cbind(dat,i,y)
indx <- (i>=0)
k <- ncol(dat)
dat <- as.matrix(dat[indx%*%matrix(1,1,k)>0])
dat <- matrix(dat,nrow(dat)/k,k)

indx <- order(dat[,28])
dat <- dat[indx,] 
n <- floor(nrow(dat)*(1-trm))
dat <- dat[1:n,]
retinc <- dat[,10]
i <- as.matrix(dat[,28])
y <- as.matrix(dat[,29])
dat <- dat[,1:27]


dat[,23] <- dat[,23]-dat[,20]-dat[,21]-dat[,22]
vn[23] <- "Adults"
xx <- cbind((retinc>0),retinc,dat[,12:17])
xn <- rbind("Has Retr",vn[10],as.matrix(vn[12:17]))
if (age2==1){
  xx <- cbind(xx,(dat[,17]^2))
  xn <- rbind(xn,"___Age^2")
}
region <- as.matrix(dat[,27])
r <- unique(region)
r <- as.matrix(sort(r))

xreg <- (region%*%matrix(1,1,nrow(r))==matrix(1,nrow(region),1)%*%t(r))
xx <- cbind(xx,dat[,18:26],xreg)
xn <- rbind(xn,as.matrix(vn[18:26]),rname)
mx <- matrix(1,n,1)%*%t(as.matrix(colMeans(xx)))
if (noinc==1){
  i0 <- (i==0)
  xx <- cbind(i0,xx)
  xn <- rbind("No Income",xn)
}

cat ("************************************************************","\n")
cat ("\n")
cat ("\n")
cat (sample,"households","\n")
cat ("Dependent Variable:            ",yname,"\n")
cat ("Trimmed Top Quantile of Income ",trm,"\n")
if (noinc==1) cat ("Number of Zero Income          ",sum(i0),"\n")
cat ("Observations                   ", n,"\n")
cat ("__________________________________________________","\n")

#*************************************************#
# Linear Model #
x <- cbind(i,xx)
beta <- qr.solve(x,y)
e <- y - x%*%beta
xname <- rbind("Income",xn)
cat ("Linear Model","\n")
out <- report(y,x,e,beta,xname)


#*************************************************#
# Polynomial Model #
p <- 9
xp1 <- cbind(i^1,i^2,i^3,i^4,i^5,i^6,i^7,i^8,i^9)  
xp2 <- cbind(10000^0,10000^1,10000^2,10000^3,10000^4,10000^5,10000^6,10000^7,10000^8)  
xp <- xp1/(matrix(1,nrow(xp1),1)%*%xp2)
x <- cbind(xp,xx)
beta <- qr.solve(x,y)
e <- y - x%*%beta
trp <- xp%*%as.matrix(beta[1:p])+mx%*%beta[(p+1+noinc):nrow(beta)]

xname <- "Income"
for (jp in 2:p){
  xjp <- paste("Income^",jp,sep="")   
  xname <- rbind(xname,xjp)
}

xname <- rbind(xname,xn)
cat ("Polynomial Model","\n")
out <- report(y,x,e,beta,xname)


#*************************************************#

xm <- xx%*%solve(t(xx)%*%xx)  
e0 <- y - xm%*%(t(xx)%*%y)
ee0 <- t(e0)%*%e0
q <- i
if (qn == 0){
  q <- as.matrix(unique(q[floor(per*n):(n-floor(per*n))]))
  qn <- nrow(q)
}else{
  q <- as.matrix(q[floor(seq(per,per+(qn-1)*(1-per*2)/qn,(1-per*2)/qn)*n)])
}

#*************************************************#
# Linear Spline Model 3

sse <- matrix(0,qn,1) 
for (j in 1:qn){
  qj <- q[j]
  ic <- i - qj
  i1 <- (i<=qj)
  i2 <- 1-i1
  dx <- cbind((ic*i1),(ic*i2))
  if (quad==1) dx <- cbind(dx,(((ic^2)*i2)/(10^6)))
  xv <- dx - xm%*%(t(xx)%*%dx)
  et <- e0 - xv%*%qr.solve(xv,e0)
  sse[j] <- t(et)%*%et
  if (round(j/10)==(j/10)) cat (j,qj,(sse[j]/n),"\n")
}

jmin <- which.min(sse)
knot <- q[jmin]

ic <- i - knot
i1 <- (i<=knot)
i2 <- 1-i1
dx <- cbind((ic*i1),(ic*i2))
if (quad==1) dx <- cbind(dx,(((ic^2)*i2)/(10^6)))
x <- cbind(dx,xx)
beta <- qr.solve(x,y)
e <- y - x%*%beta
xh <- -beta[1]*i1-beta[2]*i2
if (quad==1) xh <- xh-(ic*i2)*(2*beta[3])
x <- cbind(x,xh)
tr1 <- dx%*%beta[1:(2+quad)]+mx%*%beta[(3+quad+noinc):nrow(beta)]

xname <- rbind("Slope_1","Slope_2")
if (quad==1) xname <- rbind(xname,"Quad_2")
xname <- rbind(xname,xn,"Threshold")
cat ("\n")
cat ("\n")
if (quad==0) cat ("Linear Spline Model","\n")
if (quad==1) cat ("Linear/Quadratic Spline Model","\n")
cat ("# Quantiles Used           ", qn,"\n")
cat ("Income Threshold Quantile :", mean(i<=knot), "\n")
out <- report(y,x,e,rbind(beta,knot),xname)

out <- gr_tr(i,cbind(trp,tr1))

#*************************************************#

if (save==1){
  tr <- cbind(i,trp,tr1)
  if (smp==1) write.table(tr,file = "trans_1.dat",quote = FALSE,row.names = FALSE,col.names = FALSE)
  if (smp==2) write.table(tr,file = "trans_1.dat",quote = FALSE,row.names = FALSE,col.names = FALSE)
}

smp <- smp+1
}


